package com.cicdi.notify.util.script.engine.spel;

import com.cicdi.notify.util.script.engine.ExecuteResult;
import com.cicdi.notify.util.script.engine.ListenerSupportEngine;
import com.cicdi.notify.util.script.engine.ScriptContext;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class SpelParserEngine extends ListenerSupportEngine {

    protected final Map<String, SpelScriptContext> scriptContextCache = new ConcurrentHashMap<>();

    protected final ExpressionParser expressionParser = new SpelExpressionParser();

    @Override
    public boolean compiled(String id) {
        return scriptContextCache.containsKey(id);
    }

    @Override
    public void init(String... contents) {
    }

    @Override
    public boolean remove(String id) {
        return scriptContextCache.remove(id) != null;
    }

    @Override
    public ScriptContext getContext(String id) {
        return scriptContextCache.get(id);
    }

    @Override
    public boolean compile(String id, String code) {
        scriptContextCache.put(id, new SpelScriptContext(id, DigestUtils.md5Hex(code), expressionParser.parseExpression(code)));
        return false;
    }

    @Override
    public ExecuteResult execute(String id, Map<String, Object> param) {
        ExecuteResult result = new ExecuteResult();
        SpelScriptContext scriptContext = scriptContextCache.get(id);
        try {
            if (scriptContext != null) {
                param = new HashMap<>(param);
                param.putAll(getGlobalVariable());

                StandardEvaluationContext context = new StandardEvaluationContext(param);
                context.setVariable("u", new Date());
                for (Map.Entry<String, Object> entry : param.entrySet()) {
                    context.setVariable(entry.getKey(), entry.getValue());
                }
                Expression script = scriptContext.getScript();
                Object obj = script.getValue(context);
                result.setSuccess(true);
                result.setResult(obj);
            } else {
                result.setSuccess(false);
                result.setResult(null);
            }
        } catch (Exception ignored) {
        }
        return result;
    }

    static class SpelScriptContext extends ScriptContext {
        private final Expression script;

        public SpelScriptContext(String id, String md5, Expression script) {
            super(id, md5);
            this.script = script;
        }

        public Expression getScript() {
            return script;
        }
    }

}
