package com.cicdi.notify.dingtalk;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cicdi.notify.AbstractNotifier;
import com.cicdi.notify.DefaultNotifyType;
import com.cicdi.notify.NotifyType;
import com.cicdi.notify.Provider;
import com.cicdi.notify.template.TemplateManager;
import org.apache.http.NameValuePair;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.net.URISyntaxException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

public class DingTalkNotifier extends AbstractNotifier<DingTalkMessageTemplate> {

    private final AtomicReference<String> accessToken = new AtomicReference<>();

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://oapi.dingtalk.com/gettoken";

    private static final String notifyApi = "https://oapi.dingtalk.com/topapi/message/corpconversation/asyncsend_v2";

    private final DingTalkProperties properties;

    private final String notifierId;

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    public DingTalkNotifier(String id, DingTalkProperties properties, TemplateManager templateManager) {
        super(templateManager);
        this.properties = properties;
        this.notifierId = id;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.dingTalk;
    }

    @Override
    public Provider getProvider() {
        return DingTalkProvider.dingTalkMessage;
    }

    @Override
    public void send(DingTalkMessageTemplate template, Map<String, Object> context) {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("access_token", getToken()));

        try {
            HttpPost httpPost = new HttpPost(new URIBuilder(notifyApi).setParameters(params).build());
            String formInserter = template.createFormInserter(context);
            StringEntity entity = new StringEntity(template.createFormInserter(context));
            httpPost.setEntity(entity);
            CloseableHttpResponse response = httpClient.execute(httpPost);
            JSONObject responseJson = JSON.parseObject(EntityUtils.toString(response.getEntity()));
            System.out.println(responseJson);
        } catch (URISyntaxException | IOException e) {
            e.printStackTrace();
        }
    }

    private void checkResult(String msg) {
        if ("0".equals(msg)) {
//                log.info("发送钉钉通知成功");
        } else {
//                log.warn("发送钉钉通知失败:{}", map);
//                throw new BusinessException("发送钉钉通知失败:" + map.get("errmsg"), code);
        }
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken.get() == null) {
            return requestToken();
        }
        return accessToken.get();
    }

    private String requestToken() {
        CloseableHttpClient httpClient = HttpClientBuilder.create().build();
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("appkey", properties.getAppKey()));
        params.add(new BasicNameValuePair("appsecret", properties.getAppSecret()));
        try {
            HttpGet httpGet = new HttpGet(new URIBuilder(tokenApi).setParameters(params).build());
            CloseableHttpResponse response = httpClient.execute(httpGet);
            JSONObject responseJson = JSON.parseObject(EntityUtils.toString(response.getEntity()));
            if (responseJson.containsKey("access_token")) {
                String access_token = responseJson.get("access_token").toString();
                refreshTokenTime = System.currentTimeMillis();
                accessToken.set(access_token);
                return access_token;
            } else {
                // throw new BusinessException("获取Token失败:" + map.get("errmsg"), String.valueOf(map.get("errcode")));
            }
        } catch (URISyntaxException | IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void close() {
        accessToken.set(null);
        refreshTokenTime = 0;
    }
}
